@@ -2,7 +2,7 @@
 Generalized Linear Models with Exponential Dispersion Family
 """
 
-# Author: Christian Lorentzen <lorentzen.ch@googlemail.com>
+# Author: Christian Lorentzen <lorentzen.ch@gmail.com>
 # some parts and tricks stolen from other sklearn files.
 # License: BSD 3 clause
 
@@ -11,56 +11,42 @@ import numbers
 import numpy as np
 import scipy.optimize
 
+from ..._loss.glm_distribution import TweedieDistribution
+from ..._loss.loss import (
+    HalfGammaLoss,
+    HalfPoissonLoss,
+    HalfSquaredError,
+    HalfTweedieLoss,
+    HalfTweedieLossIdentity,
+)
 from ...base import BaseEstimator, RegressorMixin
 from ...utils.optimize import _check_optimize_result
+from ...utils import check_scalar, check_array, deprecated
 from ...utils.validation import check_is_fitted, _check_sample_weight
-from ..._loss.glm_distribution import (
-    ExponentialDispersionModel,
-    TweedieDistribution,
-    EDM_DISTRIBUTIONS,
-)
-from .link import (
-    BaseLink,
-    IdentityLink,
-    LogLink,
-)
+from ...utils._openmp_helpers import _openmp_effective_n_threads
+from .._linear_loss import LinearModelLoss
 
 
-def _safe_lin_pred(X, coef):
-    """Compute the linear predictor taking care if intercept is present."""
-    if coef.size == X.shape[1] + 1:
-        return X @ coef[1:] + coef[0]
-    else:
-        return X @ coef
-
-
-def _y_pred_deviance_derivative(coef, X, y, weights, family, link):
-    """Compute y_pred and the derivative of the deviance w.r.t coef."""
-    lin_pred = _safe_lin_pred(X, coef)
-    y_pred = link.inverse(lin_pred)
-    d1 = link.inverse_derivative(lin_pred)
-    temp = d1 * family.deviance_derivative(y, y_pred, weights)
-    if coef.size == X.shape[1] + 1:
-        devp = np.concatenate(([temp.sum()], temp @ X))
-    else:
-        devp = temp @ X  # same as X.T @ temp
-    return y_pred, devp
-
-
-class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
+class _GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
     """Regression via a penalized Generalized Linear Model (GLM).
 
-    GLMs based on a reproductive Exponential Dispersion Model (EDM) aim at
-    fitting and predicting the mean of the target y as y_pred=h(X*w).
-    Therefore, the fit minimizes the following objective function with L2
-    priors as regularizer::
+    GLMs based on a reproductive Exponential Dispersion Model (EDM) aim at fitting and
+    predicting the mean of the target y as y_pred=h(X*w) with coefficients w.
+    Therefore, the fit minimizes the following objective function with L2 priors as
+    regularizer::
 
-            1/(2*sum(s)) * deviance(y, h(X*w); s)
-            + 1/2 * alpha * |w|_2
+        1/(2*sum(s_i)) * sum(s_i * deviance(y_i, h(x_i*w)) + 1/2 * alpha * ||w||_2^2
 
-    with inverse link function h and s=sample_weight.
+    with inverse link function h, s=sample_weight and per observation (unit) deviance
+    deviance(y_i, h(x_i*w)). Note that for an EDM, 1/2 * deviance is the negative
+    log-likelihood up to a constant (in w) term.
     The parameter ``alpha`` corresponds to the lambda parameter in glmnet.
 
+    Instead of implementing the EDM family and a link function seperately, we directly
+    use the loss functions `from sklearn._loss` which have the link functions included
+    in them for performance reasons. We pick the loss functions that implement
+    (1/2 times) EDM deviances.
+
     Read more in the :ref:`User Guide <Generalized_linear_regression>`.
 
     .. versionadded:: 0.23
@@ -72,25 +58,12 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
         regularization strength. ``alpha = 0`` is equivalent to unpenalized
         GLMs. In this case, the design matrix `X` must have full column rank
         (no collinearities).
+        Values must be in the range `[0.0, inf)`.
 
     fit_intercept : bool, default=True
         Specifies if a constant (a.k.a. bias or intercept) should be
         added to the linear predictor (X @ coef + intercept).
 
-    family : {'normal', 'poisson', 'gamma', 'inverse-gaussian'} \
-            or an ExponentialDispersionModel instance, default='normal'
-        The distributional assumption of the GLM, i.e. which distribution from
-        the EDM, specifies the loss function to be minimized.
-
-    link : {'auto', 'identity', 'log'} or an instance of class BaseLink, \
-            default='auto'
-        The link function of the GLM, i.e. mapping from linear predictor
-        `X @ coeff + intercept` to prediction `y_pred`. Option 'auto' sets
-        the link depending on the chosen family as follows:
-
-        - 'identity' for Normal distribution
-        - 'log' for Poisson,  Gamma and Inverse Gaussian distributions
-
     solver : 'lbfgs', default='lbfgs'
         Algorithm to use in the optimization problem:
 
@@ -99,12 +72,14 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
 
     max_iter : int, default=100
         The maximal number of iterations for the solver.
+        Values must be in the range `[1, inf)`.
 
     tol : float, default=1e-4
         Stopping criterion. For the lbfgs solver,
         the iteration will stop when ``max{|g_j|, j = 1, ..., d} <= tol``
         where ``g_j`` is the j-th component of the gradient (derivative) of
         the objective function.
+        Values must be in the range `(0.0, inf)`.
 
     warm_start : bool, default=False
         If set to ``True``, reuse the solution of the previous call to ``fit``
@@ -112,6 +87,7 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
 
     verbose : int, default=0
         For the lbfgs solver set verbose to any positive number for verbosity.
+        Values must be in the range `[0, inf)`.
 
     Attributes
     ----------
@@ -124,6 +100,26 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
 
     n_iter_ : int
         Actual number of iterations used in the solver.
+
+    _base_loss : BaseLoss, default=HalfSquaredError()
+        This is set during fit via `self._get_loss()`.
+        A `_base_loss` contains a specific loss function as well as the link
+        function. The loss to be minimized specifies the distributional assumption of
+        the GLM, i.e. the distribution from the EDM. Here are some examples:
+
+        =======================  ========  ==========================
+        _base_loss               Link      Target Domain
+        =======================  ========  ==========================
+        HalfSquaredError         identity  y any real number
+        HalfPoissonLoss          log       0 <= y
+        HalfGammaLoss            log       0 < y
+        HalfTweedieLoss          log       dependend on tweedie power
+        HalfTweedieLossIdentity  identity  dependend on tweedie power
+        =======================  ========  ==========================
+
+        The link function of the GLM, i.e. mapping from linear predictor
+        `X @ coeff + intercept` to prediction `y_pred`. For instance, with a log link,
+        we have `y_pred = exp(X @ coeff + intercept)`.
     """
 
     def __init__(
@@ -131,8 +127,6 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
         *,
         alpha=1.0,
         fit_intercept=True,
-        family="normal",
-        link="auto",
         solver="lbfgs",
         max_iter=100,
         tol=1e-4,
@@ -141,8 +135,6 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
     ):
         self.alpha = alpha
         self.fit_intercept = fit_intercept
-        self.family = family
-        self.link = link
         self.solver = solver
         self.max_iter = max_iter
         self.tol = tol
@@ -168,53 +160,13 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
         self : object
             Fitted model.
         """
-        if isinstance(self.family, ExponentialDispersionModel):
-            self._family_instance = self.family
-        elif self.family in EDM_DISTRIBUTIONS:
-            self._family_instance = EDM_DISTRIBUTIONS[self.family]()
-        else:
-            raise ValueError(
-                "The family must be an instance of class"
-                " ExponentialDispersionModel or an element of"
-                " ['normal', 'poisson', 'gamma', 'inverse-gaussian']"
-                "; got (family={0})".format(self.family)
-            )
-
-        # Guarantee that self._link_instance is set to an instance of
-        # class BaseLink
-        if isinstance(self.link, BaseLink):
-            self._link_instance = self.link
-        else:
-            if self.link == "auto":
-                if isinstance(self._family_instance, TweedieDistribution):
-                    if self._family_instance.power <= 0:
-                        self._link_instance = IdentityLink()
-                    if self._family_instance.power >= 1:
-                        self._link_instance = LogLink()
-                else:
-                    raise ValueError(
-                        "No default link known for the "
-                        "specified distribution family. Please "
-                        "set link manually, i.e. not to 'auto'; "
-                        "got (link='auto', family={})".format(self.family)
-                    )
-            elif self.link == "identity":
-                self._link_instance = IdentityLink()
-            elif self.link == "log":
-                self._link_instance = LogLink()
-            else:
-                raise ValueError(
-                    "The link must be an instance of class Link or "
-                    "an element of ['auto', 'identity', 'log']; "
-                    "got (link={0})".format(self.link)
-                )
-
-        if not isinstance(self.alpha, numbers.Number) or self.alpha < 0:
-            raise ValueError(
-                "Penalty term must be a non-negative number; got (alpha={0})".format(
-                    self.alpha
-                )
-            )
+        check_scalar(
+            self.alpha,
+            name="alpha",
+            target_type=numbers.Real,
+            min_val=0.0,
+            include_boundaries="left",
+        )
         if not isinstance(self.fit_intercept, bool):
             raise ValueError(
                 "The argument fit_intercept must be bool; got {0}".format(
@@ -223,29 +175,34 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
             )
         if self.solver not in ["lbfgs"]:
             raise ValueError(
-                "GeneralizedLinearRegressor supports only solvers"
-                "'lbfgs'; got {0}".format(self.solver)
+                f"{self.__class__.__name__} supports only solvers 'lbfgs'; "
+                f"got {self.solver}"
             )
         solver = self.solver
-        if not isinstance(self.max_iter, numbers.Integral) or self.max_iter <= 0:
-            raise ValueError(
-                "Maximum number of iteration must be a positive "
-                "integer;"
-                " got (max_iter={0!r})".format(self.max_iter)
-            )
-        if not isinstance(self.tol, numbers.Number) or self.tol <= 0:
-            raise ValueError(
-                "Tolerance for stopping criteria must be "
-                "positive; got (tol={0!r})".format(self.tol)
-            )
+        check_scalar(
+            self.max_iter,
+            name="max_iter",
+            target_type=numbers.Integral,
+            min_val=1,
+        )
+        check_scalar(
+            self.tol,
+            name="tol",
+            target_type=numbers.Real,
+            min_val=0.0,
+            include_boundaries="neither",
+        )
+        check_scalar(
+            self.verbose,
+            name="verbose",
+            target_type=numbers.Integral,
+            min_val=0,
+        )
         if not isinstance(self.warm_start, bool):
             raise ValueError(
                 "The argument warm_start must be bool; got {0}".format(self.warm_start)
             )
 
-        family = self._family_instance
-        link = self._link_instance
-
         X, y = self._validate_data(
             X,
             y,
@@ -255,57 +212,71 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
             multi_output=False,
         )
 
-        weights = _check_sample_weight(sample_weight, X)
+        # required by losses
+        if solver == "lbfgs":
+            # lbfgs will force coef and therefore raw_prediction to be float64. The
+            # base_loss needs y, X @ coef and sample_weight all of same dtype
+            # (and contiguous).
+            loss_dtype = np.float64
+        else:
+            loss_dtype = min(max(y.dtype, X.dtype), np.float64)
+        y = check_array(y, dtype=loss_dtype, order="C", ensure_2d=False)
 
-        _, n_features = X.shape
+        # TODO: We could support samples_weight=None as the losses support it.
+        # Note that _check_sample_weight calls check_array(order="C") required by
+        # losses.
+        sample_weight = _check_sample_weight(sample_weight, X, dtype=loss_dtype)
 
-        if not np.all(family.in_y_range(y)):
+        n_samples, n_features = X.shape
+        self._base_loss = self._get_loss()
+
+        linear_loss = LinearModelLoss(
+            base_loss=self._base_loss,
+            fit_intercept=self.fit_intercept,
+        )
+
+        if not linear_loss.base_loss.in_y_true_range(y):
             raise ValueError(
-                "Some value(s) of y are out of the valid range for family {0}".format(
-                    family.__class__.__name__
-                )
+                "Some value(s) of y are out of the valid range of the loss"
+                f" {self._base_loss.__class__.__name__!r}."
             )
+
         # TODO: if alpha=0 check that X is not rank deficient
 
-        # rescaling of sample_weight
-        #
-        # IMPORTANT NOTE: Since we want to minimize
-        # 1/(2*sum(sample_weight)) * deviance + L2,
-        # deviance = sum(sample_weight * unit_deviance),
-        # we rescale weights such that sum(weights) = 1 and this becomes
-        # 1/2*deviance + L2 with deviance=sum(weights * unit_deviance)
-        weights = weights / weights.sum()
+        # IMPORTANT NOTE: Rescaling of sample_weight:
+        # We want to minimize
+        #     obj = 1/(2*sum(sample_weight)) * sum(sample_weight * deviance)
+        #         + 1/2 * alpha * L2,
+        # with
+        #     deviance = 2 * loss.
+        # The objective is invariant to multiplying sample_weight by a constant. We
+        # choose this constant such that sum(sample_weight) = 1. Thus, we end up with
+        #     obj = sum(sample_weight * loss) + 1/2 * alpha * L2.
+        # Note that LinearModelLoss.loss() computes sum(sample_weight * loss).
+        sample_weight = sample_weight / sample_weight.sum()
 
         if self.warm_start and hasattr(self, "coef_"):
             if self.fit_intercept:
-                coef = np.concatenate((np.array([self.intercept_]), self.coef_))
+                # LinearModelLoss needs intercept at the end of coefficient array.
+                coef = np.concatenate((self.coef_, np.array([self.intercept_])))
             else:
                 coef = self.coef_
+            coef = coef.astype(loss_dtype, copy=False)
         else:
             if self.fit_intercept:
-                coef = np.zeros(n_features + 1)
-                coef[0] = link(np.average(y, weights=weights))
-            else:
-                coef = np.zeros(n_features)
-
-        # algorithms for optimization
-
-        if solver == "lbfgs":
-
-            def func(coef, X, y, weights, alpha, family, link):
-                y_pred, devp = _y_pred_deviance_derivative(
-                    coef, X, y, weights, family, link
+                coef = np.zeros(n_features + 1, dtype=loss_dtype)
+                coef[-1] = linear_loss.base_loss.link.link(
+                    np.average(y, weights=sample_weight)
                 )
-                dev = family.deviance(y, y_pred, weights)
-                # offset if coef[0] is intercept
-                offset = 1 if self.fit_intercept else 0
-                coef_scaled = alpha * coef[offset:]
-                obj = 0.5 * dev + 0.5 * (coef[offset:] @ coef_scaled)
-                objp = 0.5 * devp
-                objp[offset:] += coef_scaled
-                return obj, objp
+            else:
+                coef = np.zeros(n_features, dtype=loss_dtype)
 
-            args = (X, y, weights, self.alpha, family, link)
+        # Algorithms for optimization:
+        # Note again that our losses implement 1/2 * deviance.
+        if solver == "lbfgs":
+            func = linear_loss.loss_gradient
+            l2_reg_strength = self.alpha
+            n_threads = _openmp_effective_n_threads()
 
             opt_res = scipy.optimize.minimize(
                 func,
@@ -318,14 +289,14 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
                     "gtol": self.tol,
                     "ftol": 1e3 * np.finfo(float).eps,
                 },
-                args=args,
+                args=(X, y, sample_weight, l2_reg_strength, n_threads),
             )
             self.n_iter_ = _check_optimize_result("lbfgs", opt_res)
             coef = opt_res.x
 
         if self.fit_intercept:
-            self.intercept_ = coef[0]
-            self.coef_ = coef[1:]
+            self.intercept_ = coef[-1]
+            self.coef_ = coef[:-1]
         else:
             # set intercept to zero as the other linear models do
             self.intercept_ = 0.0
@@ -336,6 +307,8 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
     def _linear_predictor(self, X):
         """Compute the linear_predictor = `X @ coef_ + intercept_`.
 
+        Note that we often use the term raw_prediction instead of linear predictor.
+
         Parameters
         ----------
         X : {array-like, sparse matrix} of shape (n_samples, n_features)
@@ -371,16 +344,16 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
             Returns predicted values.
         """
         # check_array is done in _linear_predictor
-        eta = self._linear_predictor(X)
-        y_pred = self._link_instance.inverse(eta)
+        raw_prediction = self._linear_predictor(X)
+        y_pred = self._base_loss.link.inverse(raw_prediction)
         return y_pred
 
     def score(self, X, y, sample_weight=None):
         """Compute D^2, the percentage of deviance explained.
 
         D^2 is a generalization of the coefficient of determination R^2.
-        R^2 uses squared error and D^2 deviance. Note that those two are equal
-        for ``family='normal'``.
+        R^2 uses squared error and D^2 uses the deviance of this GLM, see the
+        :ref:`User Guide <regression_metrics>`.
 
         D^2 is defined as
         :math:`D^2 = 1-\\frac{D(y_{true},y_{pred})}{D_{null}}`,
@@ -406,30 +379,88 @@ class GeneralizedLinearRegressor(RegressorMixin, BaseEstimator):
         score : float
             D^2 of self.predict(X) w.r.t. y.
         """
+        # TODO: Adapt link to User Guide in the docstring, once
+        # https://github.com/scikit-learn/scikit-learn/pull/22118 is merged.
+        #
         # Note, default score defined in RegressorMixin is R^2 score.
         # TODO: make D^2 a score function in module metrics (and thereby get
         #       input validation and so on)
-        weights = _check_sample_weight(sample_weight, X)
-        y_pred = self.predict(X)
-        dev = self._family_instance.deviance(y, y_pred, weights=weights)
-        y_mean = np.average(y, weights=weights)
-        dev_null = self._family_instance.deviance(y, y_mean, weights=weights)
-        return 1 - dev / dev_null
+        raw_prediction = self._linear_predictor(X)  # validates X
+        # required by losses
+        y = check_array(y, dtype=raw_prediction.dtype, order="C", ensure_2d=False)
+
+        if sample_weight is not None:
+            # Note that _check_sample_weight calls check_array(order="C") required by
+            # losses.
+            sample_weight = _check_sample_weight(sample_weight, X, dtype=y.dtype)
+
+        base_loss = self._base_loss
+
+        if not base_loss.in_y_true_range(y):
+            raise ValueError(
+                "Some value(s) of y are out of the valid range of the loss"
+                f" {base_loss.__name__}."
+            )
+
+        # Note that constant_to_optimal_zero is already multiplied by sample_weight.
+        constant = np.mean(base_loss.constant_to_optimal_zero(y_true=y))
+        if sample_weight is not None:
+            constant *= sample_weight.shape[0] / np.sum(sample_weight)
+
+        # Missing factor of 2 in deviance cancels out.
+        deviance = base_loss(
+            y_true=y,
+            raw_prediction=raw_prediction,
+            sample_weight=sample_weight,
+            n_threads=1,
+        )
+        y_mean = base_loss.link.link(np.average(y, weights=sample_weight))
+        deviance_null = base_loss(
+            y_true=y,
+            raw_prediction=np.tile(y_mean, y.shape[0]),
+            sample_weight=sample_weight,
+            n_threads=1,
+        )
+        return 1 - (deviance + constant) / (deviance_null + constant)
 
     def _more_tags(self):
-        # create the _family_instance if fit wasn't called yet.
-        if hasattr(self, "_family_instance"):
-            _family_instance = self._family_instance
-        elif isinstance(self.family, ExponentialDispersionModel):
-            _family_instance = self.family
-        elif self.family in EDM_DISTRIBUTIONS:
-            _family_instance = EDM_DISTRIBUTIONS[self.family]()
+        # Create instance of BaseLoss if fit wasn't called yet. This is necessary as
+        # TweedieRegressor might set the used loss during fit different from
+        # self._base_loss.
+        base_loss = self._get_loss()
+        return {"requires_positive_y": not base_loss.in_y_true_range(-1.0)}
+
+    def _get_loss(self):
+        """This is only necessary because of the link and power arguments of the
+        TweedieRegressor.
+
+        Note that we do not need to pass sample_weight to the loss class as this is
+        only needed to set loss.constant_hessian on which GLMs do not rely.
+        """
+        return HalfSquaredError()
+
+    # TODO(1.3): remove
+    @deprecated(  # type: ignore
+        "Attribute `family` was deprecated in version 1.1 and will be removed in 1.3."
+    )
+    @property
+    def family(self):
+        """Ensure backward compatibility for the time of deprecation."""
+        if isinstance(self, PoissonRegressor):
+            return "poisson"
+        elif isinstance(self, GammaRegressor):
+            return "gamma"
+        elif isinstance(self, TweedieRegressor):
+            return TweedieDistribution(power=self.power)
         else:
-            raise ValueError
-        return {"requires_positive_y": not _family_instance.in_y_range(-1.0)}
+            raise ValueError(  # noqa
+                "This should never happen. You presumably accessed the deprecated "
+                "`family` attribute from a subclass of the private scikit-learn class "
+                "_GeneralizedLinearRegressor."
+            )
 
 
-class PoissonRegressor(GeneralizedLinearRegressor):
+class PoissonRegressor(_GeneralizedLinearRegressor):
     """Generalized Linear Model with a Poisson distribution.
 
     This regressor uses the 'log' link function.
@@ -445,6 +476,7 @@ class PoissonRegressor(GeneralizedLinearRegressor):
         regularization strength. ``alpha = 0`` is equivalent to unpenalized
         GLMs. In this case, the design matrix `X` must have full column rank
         (no collinearities).
+        Values must be in the range `[0.0, inf)`.
 
     fit_intercept : bool, default=True
         Specifies if a constant (a.k.a. bias or intercept) should be
@@ -452,12 +484,14 @@ class PoissonRegressor(GeneralizedLinearRegressor):
 
     max_iter : int, default=100
         The maximal number of iterations for the solver.
+        Values must be in the range `[1, inf)`.
 
     tol : float, default=1e-4
         Stopping criterion. For the lbfgs solver,
         the iteration will stop when ``max{|g_j|, j = 1, ..., d} <= tol``
         where ``g_j`` is the j-th component of the gradient (derivative) of
         the objective function.
+        Values must be in the range `(0.0, inf)`.
 
     warm_start : bool, default=False
         If set to ``True``, reuse the solution of the previous call to ``fit``
@@ -465,6 +499,7 @@ class PoissonRegressor(GeneralizedLinearRegressor):
 
     verbose : int, default=0
         For the lbfgs solver set verbose to any positive number for verbosity.
+        Values must be in the range `[0, inf)`.
 
     Attributes
     ----------
@@ -489,8 +524,12 @@ class PoissonRegressor(GeneralizedLinearRegressor):
     n_iter_ : int
         Actual number of iterations used in the solver.
 
+    See Also
+    --------
+    TweedieRegressor : Generalized Linear Model with a Tweedie distribution.
+
     Examples
-    ----------
+    --------
     >>> from sklearn import linear_model
     >>> clf = linear_model.PoissonRegressor()
     >>> X = [[1, 2], [2, 3], [3, 4], [4, 3]]
@@ -505,11 +544,6 @@ class PoissonRegressor(GeneralizedLinearRegressor):
     2.088...
     >>> clf.predict([[1, 1], [3, 4]])
     array([10.676..., 21.875...])
-
-    See Also
-    ----------
-    GeneralizedLinearRegressor : Generalized Linear Model with a Poisson
-        distribution.
     """
 
     def __init__(
@@ -522,31 +556,20 @@ class PoissonRegressor(GeneralizedLinearRegressor):
         warm_start=False,
         verbose=0,
     ):
-
         super().__init__(
             alpha=alpha,
             fit_intercept=fit_intercept,
-            family="poisson",
-            link="log",
             max_iter=max_iter,
             tol=tol,
             warm_start=warm_start,
             verbose=verbose,
         )
 
-    @property
-    def family(self):
-        """Return the string `'poisson'`."""
-        # Make this attribute read-only to avoid mis-uses e.g. in GridSearch.
-        return "poisson"
-
-    @family.setter
-    def family(self, value):
-        if value != "poisson":
-            raise ValueError("PoissonRegressor.family must be 'poisson'!")
+    def _get_loss(self):
+        return HalfPoissonLoss()
 
 
-class GammaRegressor(GeneralizedLinearRegressor):
+class GammaRegressor(_GeneralizedLinearRegressor):
     """Generalized Linear Model with a Gamma distribution.
 
     This regressor uses the 'log' link function.
@@ -562,6 +585,7 @@ class GammaRegressor(GeneralizedLinearRegressor):
         regularization strength. ``alpha = 0`` is equivalent to unpenalized
         GLMs. In this case, the design matrix `X` must have full column rank
         (no collinearities).
+        Values must be in the range `[0.0, inf)`.
 
     fit_intercept : bool, default=True
         Specifies if a constant (a.k.a. bias or intercept) should be
@@ -569,12 +593,14 @@ class GammaRegressor(GeneralizedLinearRegressor):
 
     max_iter : int, default=100
         The maximal number of iterations for the solver.
+        Values must be in the range `[1, inf)`.
 
     tol : float, default=1e-4
         Stopping criterion. For the lbfgs solver,
         the iteration will stop when ``max{|g_j|, j = 1, ..., d} <= tol``
         where ``g_j`` is the j-th component of the gradient (derivative) of
         the objective function.
+        Values must be in the range `(0.0, inf)`.
 
     warm_start : bool, default=False
         If set to ``True``, reuse the solution of the previous call to ``fit``
@@ -582,6 +608,7 @@ class GammaRegressor(GeneralizedLinearRegressor):
 
     verbose : int, default=0
         For the lbfgs solver set verbose to any positive number for verbosity.
+        Values must be in the range `[0, inf)`.
 
     Attributes
     ----------
@@ -639,31 +666,20 @@ class GammaRegressor(GeneralizedLinearRegressor):
         warm_start=False,
         verbose=0,
     ):
-
         super().__init__(
             alpha=alpha,
             fit_intercept=fit_intercept,
-            family="gamma",
-            link="log",
             max_iter=max_iter,
             tol=tol,
             warm_start=warm_start,
             verbose=verbose,
         )
 
-    @property
-    def family(self):
-        """Return the family of the regressor."""
-        # Make this attribute read-only to avoid mis-uses e.g. in GridSearch.
-        return "gamma"
-
-    @family.setter
-    def family(self, value):
-        if value != "gamma":
-            raise ValueError("GammaRegressor.family must be 'gamma'!")
+    def _get_loss(self):
+        return HalfGammaLoss()
 
 
-class TweedieRegressor(GeneralizedLinearRegressor):
+class TweedieRegressor(_GeneralizedLinearRegressor):
     """Generalized Linear Model with a Tweedie distribution.
 
     This estimator can be used to model different GLMs depending on the
@@ -700,6 +716,7 @@ class TweedieRegressor(GeneralizedLinearRegressor):
         regularization strength. ``alpha = 0`` is equivalent to unpenalized
         GLMs. In this case, the design matrix `X` must have full column rank
         (no collinearities).
+        Values must be in the range `[0.0, inf)`.
 
     fit_intercept : bool, default=True
         Specifies if a constant (a.k.a. bias or intercept) should be
@@ -708,19 +725,22 @@ class TweedieRegressor(GeneralizedLinearRegressor):
     link : {'auto', 'identity', 'log'}, default='auto'
         The link function of the GLM, i.e. mapping from linear predictor
         `X @ coeff + intercept` to prediction `y_pred`. Option 'auto' sets
-        the link depending on the chosen family as follows:
+        the link depending on the chosen `power` parameter as follows:
 
-        - 'identity' for Normal distribution
-        - 'log' for Poisson,  Gamma and Inverse Gaussian distributions
+        - 'identity' for ``power <= 0``, e.g. for the Normal distribution
+        - 'log' for ``power > 0``, e.g. for Poisson, Gamma and Inverse Gaussian
+          distributions
 
     max_iter : int, default=100
         The maximal number of iterations for the solver.
+        Values must be in the range `[1, inf)`.
 
     tol : float, default=1e-4
         Stopping criterion. For the lbfgs solver,
         the iteration will stop when ``max{|g_j|, j = 1, ..., d} <= tol``
         where ``g_j`` is the j-th component of the gradient (derivative) of
         the objective function.
+        Values must be in the range `(0.0, inf)`.
 
     warm_start : bool, default=False
         If set to ``True``, reuse the solution of the previous call to ``fit``
@@ -728,6 +748,7 @@ class TweedieRegressor(GeneralizedLinearRegressor):
 
     verbose : int, default=0
         For the lbfgs solver set verbose to any positive number for verbosity.
+        Values must be in the range `[0, inf)`.
 
     Attributes
     ----------
@@ -752,8 +773,13 @@ class TweedieRegressor(GeneralizedLinearRegressor):
 
         .. versionadded:: 1.0
 
+    See Also
+    --------
+    PoissonRegressor : Generalized Linear Model with a Poisson distribution.
+    GammaRegressor : Generalized Linear Model with a Gamma distribution.
+
     Examples
-    ----------
+    --------
     >>> from sklearn import linear_model
     >>> clf = linear_model.TweedieRegressor()
     >>> X = [[1, 2], [2, 3], [3, 4], [4, 3]]
@@ -782,32 +808,31 @@ class TweedieRegressor(GeneralizedLinearRegressor):
         warm_start=False,
         verbose=0,
     ):
-
         super().__init__(
             alpha=alpha,
             fit_intercept=fit_intercept,
-            family=TweedieDistribution(power=power),
-            link=link,
             max_iter=max_iter,
             tol=tol,
             warm_start=warm_start,
             verbose=verbose,
         )
+        self.link = link
+        self.power = power
 
-    @property
-    def family(self):
-        # We use a property with a setter to make sure that the family is
-        # always a Tweedie distribution, and that self.power and
-        # self.family.power are identical by construction.
-        dist = TweedieDistribution(power=self.power)
-        # TODO: make the returned object immutable
-        return dist
-
-    @family.setter
-    def family(self, value):
-        if isinstance(value, TweedieDistribution):
-            self.power = value.power
+    def _get_loss(self):
+        if self.link == "auto":
+            if self.power <= 0:
+                # identity link
+                return HalfTweedieLossIdentity(power=self.power)
+            else:
+                # log link
+                return HalfTweedieLoss(power=self.power)
+        elif self.link == "log":
+            return HalfTweedieLoss(power=self.power)
+        elif self.link == "identity":
+            return HalfTweedieLossIdentity(power=self.power)
         else:
-            raise TypeError(
-                "TweedieRegressor.family must be of type TweedieDistribution!"
+            raise ValueError(
+                "The link must be an element of ['auto', 'identity', 'log']; "
+                f"got (link={self.link!r})"
             )
